import base64
import datetime
import json
import logging
import os
import ssl
import sys
import urllib.request
from urllib.error import URLError

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s %(levelname)s:%(name)s:%(message)s')


def send(labels, url=None, annotations=None):
  alert = {
      'generatorURL':
          url,  # 格式不对会报错, 可为 null or ''
      'labels':
          labels,
      'annotations':
          annotations,
      'startsAt':
          datetime.datetime.now().astimezone(
              datetime.timezone(datetime.timedelta(hours=8))).isoformat()
  }
  # auth = base64.b64encode(f"user:password".encode()).decode("ascii")
  headers = {
      'Content-type': 'application/json',
      'Authorization': f'Basic {os.environ["ALERTMANANGER_BASIC_AUTH"]}'
  }
  data = [alert]
  req = urllib.request.Request('https://alert.goodow.com/api/v2/alerts',
                               data=json.dumps(data).encode('utf-8'),
                               headers=headers)
  try:
    resp = urllib.request.urlopen(req)
  except URLError as e:
    logging.info(f'retry, reason({e.reason})')
    if isinstance(e.reason,
                  ConnectionResetError):  # [Errno 104] Connection reset by peer
      headers['Host'] = 'alert.goodow.com'
      req = urllib.request.Request('https://119.29.125.143:443/api/v2/alerts',
                                   data=json.dumps(data).encode('utf-8'),
                                   headers=headers)
      ctx = ssl.create_default_context()
      ctx.check_hostname = False
      ctx.verify_mode = ssl.CERT_NONE
      resp = urllib.request.urlopen(req, context=ctx)

  if resp.status != 200:
    print('发送 Alert 出错', resp.status, resp.reason)
    raise Exception(f'{resp.status} {resp.reason}')


if __name__ == '__main__':
  github = json.loads(os.environ['GITHUB_CONTEXT'])
  job = json.loads(os.environ['JOB_CONTEXT'])
  labels = {
      'alertname': 'github workflow',
      'status': job['status'
                   ],  # job.status 报错: 'dict' object has no attribute 'status'
      'repo': github['repository']
  }
  head_commit = github['event']['head_commit']
  url = f'{head_commit["url"]}/checks'
  annotations = {'commit': head_commit['message']}

  send(labels, url=url, annotations=annotations)
